@@ -61,17 +61,42 @@ void WeatherValidator::dataUpdated(const QString &source, const Plasma5Support::
                 ++i;
             }
         }
-
     } else if (validationResult[1] == QLatin1String("timeout")) {
         Q_EMIT error(i18n("Connection to %1 weather server timed out.", m_ionName));
-    } else {
-        const QString searchTerm = validationResult.size() > 3 ? validationResult[3] : source;
-        Q_EMIT error(i18n("Cannot find '%1' using %2.", searchTerm, m_ionName));
     }
 
     Q_EMIT finished(locationSources);
 }
 
+LocationItem::LocationItem(const QString &source)
+    : value(source)
+{
+    const QStringList sourceTerms = source.split(QLatin1Char('|'), Qt::SkipEmptyParts);
+    if (sourceTerms.count() <= 2) {
+        return;
+    }
+    weatherService = sourceTerms[0];
+    weatherStation = sourceTerms[2];
+    quality = relativeQuality(weatherService);
+}
+
+int LocationItem::relativeQuality(const QString &service) const
+{
+    static const QMap<QString, int> serviceQuality = {
+        {"wettercom", -1}, // wetter.com does not provide current weather status
+        {"bbcukmet", 0}, // only 3-day forecast and no alerts
+        {"dwd", 1}, // 7-day forecast and alerts, but some stations do not provide observation data,
+        {"noaa", 2}, // 12h 7-day forecast, observation and alerts
+        {"envcan", 2}, // 12h 7-day forecast, observation and alerts
+    };
+
+    if (!serviceQuality.contains(service)) {
+        return 1; // Fallback. Default quality
+    }
+
+    return serviceQuality.value(service);
+}
+
 LocationListModel::LocationListModel(QObject *parent)
     : QAbstractListModel(parent)
     , m_validatingInput(false)
@@ -191,24 +216,38 @@ void LocationListModel::validatorError(const QString &error)
 
 void LocationListModel::addSources(const QMap<QString, QString> &sources)
 {
-    QMapIterator<QString, QString> it(sources);
+    static QList<LocationItem> fallbackLocations;
 
     // TODO: be more elaborate and use beginInsertRows() & endInsertRows()
     beginResetModel();
 
-    while (it.hasNext()) {
-        it.next();
-        const QStringList list = it.value().split(QLatin1Char('|'), Qt::SkipEmptyParts);
-        if (list.count() > 2) {
-            qDebug() << list;
-            m_locations.append(LocationItem(list[2], list[0], it.value()));
+    for (const auto &source : sources) {
+        const auto item = LocationItem(source);
+        if (item.weatherStation.isEmpty()) {
+            continue;
+        }
+        if (item.quality >= 0) {
+            m_locations << item;
+        } else {
+            fallbackLocations << item;
         }
     }
 
+    // Promote services with better quality to the top of the list
+    std::stable_sort(m_locations.begin(), m_locations.end(), [](const auto &a, const auto &b) {
+        return a.quality >= b.quality;
+    });
+
     endResetModel();
 
     ++m_checkedInCount;
     if (m_checkedInCount >= m_validators.count()) {
+        if (m_locations.isEmpty()) {
+            beginResetModel();
+            m_locations = fallbackLocations;
+            endResetModel();
+        }
+        fallbackLocations.clear();
         completeSearch();
     }
 }
